//+------------------------------------------------------------------+
//|                                                     _ZZ test.mq5 |
//|                                         Copyright 2011, AlexSTAL |
//|                                           http://www.alexstal.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, AlexSTAL"
#property link      "http://www.alexstal.ru"
#property version   "1.00"

#include <CustPrevCalculated.mqh>
#include <ZigZags.mqh>

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_plots   1
//---- plot Zigzag
#property indicator_label1  "Zigzag"
#property indicator_type1   DRAW_ZIGZAG
#property indicator_color1  Red
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1

// Number of bars for the calculation of extremums
// cannot be less than 2
input uchar iExtPeriod = 12;
// Minimum price distance between adjacent peak and valley (otherwise it's not registered)
input uchar iMinAmplitude = 10;
// Minimum movement of price in points on the zero bar for the indicator to be recalculated
input uchar iMinMotion = 0;
// Use more precise algorithm of calculation of order of the High/Low bar forming
input bool iUseSmallerTFforEB = true;
// Number of bars
input int iBars = 100;

// Buffers of the indicator
double UP[], DN[];

CCustPrevCalculated CustPrevCalculated;
CCustZigZagPPC ZZ;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
   //--- indicator buffers mapping
   SetIndexBuffer(0, UP, INDICATOR_DATA);
   SetIndexBuffer(1, DN, INDICATOR_DATA);
   ArraySetAsSeries(UP, true);
   ArraySetAsSeries(DN, true);
   PlotIndexSetDouble(0, PLOT_EMPTY_VALUE, EMPTY_VALUE);
   PlotIndexSetDouble(1, PLOT_EMPTY_VALUE, EMPTY_VALUE);
   IndicatorSetInteger(INDICATOR_DIGITS, _Digits);
   
   CustPrevCalculated.InitData(_Symbol, _Period, iBars, CPCHSM_Normal, CPCH_high|CPCH_low, 30);
   ZZ.Init(GetPointer(CustPrevCalculated), _Symbol, _Period, iBars, CPCHSM_Normal, CPCH_high|CPCH_low, 30, iMinMotion, iUseSmallerTFforEB, iExtPeriod, iMinAmplitude);
   //---
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[])
  {
   //---
   int i;
   for (i = ((rates_total-prev_calculated)==0?0:(rates_total-prev_calculated-1)); i >= 0; i--)
     {
      UP[i] = EMPTY_VALUE;
      DN[i] = EMPTY_VALUE;
     }
   
   CPCPrepareDataResultCode resData = CustPrevCalculated.PrepareData();
   CPCPrepareDataResultCode resZZ = ZZ.PrepareData(resData);
   if ( (resZZ != CPCPDRC_NoData) && (resZZ != CPCPDRC_NoRecountNotRequired) )
      ZZ.Calculate();
   
   for (i = 0; i < (int)MathMin(ZZ.GetBarsCount(), rates_total); i++)
     {
      UP[i] = ZZ.GetUP(i, true);
      DN[i] = ZZ.GetDN(i, true);
     }
   //--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
